@@ -3,20 +3,9 @@
 #include <stdbool.h>
 #include <stdint.h>
 
-#include "nvim/func_attr.h"
 #include "nvim/option_vars.h"
 #include "nvim/strings.h"  // IWYU pragma: keep
 
-/// Return the folded-case equivalent of the given character
-///
-/// @param[in]  c  Character to transform.
-///
-/// @return Folded variant.
-#define CH_FOLD(c) \
-  utf_fold((sizeof(c) == sizeof(char)) \
-           ? ((int)(uint8_t)(c)) \
-           : ((int)(c)))
-
 /// Flags for vim_str2nr()
 typedef enum {
   STR2NR_DEC = 0,
@@ -41,15 +30,13 @@ typedef enum {
 
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "charset.h.generated.h"
+# include "charset.h.inline.generated.h"
 #endif
 
-static inline bool vim_isbreak(int c)
-  REAL_FATTR_CONST
-  REAL_FATTR_ALWAYS_INLINE;
-
 /// Check if `c` is one of the characters in 'breakat'.
 /// Used very often if 'linebreak' is set
 static inline bool vim_isbreak(int c)
+  FUNC_ATTR_CONST FUNC_ATTR_ALWAYS_INLINE
 {
   return breakat_flags[(uint8_t)c];
 }
