@@ -31,9 +31,8 @@
 //
 //    All keys not described in the above table are reserved for future use.
 //
-// 2. Add a "case: CMD_xxx" in the big switch in ex_docmd.c.
-// 3. Add an entry in the index for Ex commands at ":help ex-cmd-index".
-// 4. Add documentation in ../doc/xxx.txt.  Add a tag for both the short and
+// 2. Add an entry in the index for Ex commands at ":help ex-cmd-index".
+// 3. Add documentation in ../doc/xxx.txt.  Add a tag for both the short and
 //    long name of the command.
 
 #define EX_RANGE           0x001u  // allow a linespecs
@@ -94,35 +93,6 @@ typedef struct exarg exarg_T;
 typedef void (*ex_func_T)(exarg_T *eap);
 typedef int (*ex_preview_func_T)(exarg_T *eap, int cmdpreview_ns, handle_T cmdpreview_bufnr);
 
-// NOTE: These possible could be removed and changed so that
-// Callback could take a "command" style string, and simply
-// execute that (instead of it being a function).
-//
-// But it's still a bit weird to do that.
-//
-// Another option would be that we just make a callback reference to
-// "execute($INPUT)" or something like that, so whatever the user
-// sends in via autocmds is just executed via this.
-//
-// However, that would probably have some performance cost (probably
-// very marginal, but still some cost either way).
-typedef enum {
-  CALLABLE_NONE,
-  CALLABLE_EX,
-  CALLABLE_CB,
-} AucmdExecutableType;
-
-typedef struct aucmd_executable_t AucmdExecutable;
-struct aucmd_executable_t {
-  AucmdExecutableType type;
-  union {
-    char *cmd;
-    Callback cb;
-  } callable;
-};
-
-#define AUCMD_EXECUTABLE_INIT { .type = CALLABLE_NONE }
-
 typedef char *(*LineGetter)(int, void *, int, bool);
 
 /// Structure for command definition.
@@ -233,5 +203,5 @@ typedef struct {
 typedef struct {
   char *sub;            ///< Previous replacement string.
   Timestamp timestamp;  ///< Time when it was last set.
-  list_T *additional_elements;  ///< Additional data left from ShaDa file.
+  AdditionalData *additional_data;  ///< Additional data left from ShaDa file.
 } SubReplacementString;
