@@ -2,6 +2,7 @@
 
 #include <stdbool.h>
 #include <stdint.h>  // IWYU pragma: keep
+#include <string.h>
 #include <time.h>  // IWYU pragma: keep
 
 #include "auto/config.h"
@@ -45,8 +46,6 @@ EXTERN size_t arena_alloc_count INIT( = 0);
   ((v).capacity = (s), \
    (v).items = (void *)arena_alloc(a, sizeof((v).items[0]) * (v).capacity, true))
 
-#define ARENA_BLOCK_SIZE 4096
-
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "memory.h.generated.h"
 #endif
@@ -72,5 +71,3 @@ EXTERN size_t arena_alloc_count INIT( = 0);
 
 // Like strcpy() but allows overlapped source and destination.
 #define STRMOVE(d, s)       memmove((d), (s), strlen(s) + 1)
-
-#define STRCAT(d, s)        strcat((char *)(d), (char *)(s))  // NOLINT(runtime/printf)
