@@ -21,6 +21,7 @@
 #include "nvim/decoration_defs.h"
 #include "nvim/drawscreen.h"
 #include "nvim/edit.h"
+#include "nvim/errors.h"
 #include "nvim/eval/funcs.h"
 #include "nvim/eval/typval.h"
 #include "nvim/eval/typval_defs.h"
@@ -31,8 +32,6 @@
 #include "nvim/gettext_defs.h"
 #include "nvim/globals.h"
 #include "nvim/grid.h"
-#include "nvim/grid_defs.h"
-#include "nvim/highlight.h"
 #include "nvim/highlight_defs.h"
 #include "nvim/highlight_group.h"
 #include "nvim/macros_defs.h"
@@ -42,7 +41,6 @@
 #include "nvim/mbyte.h"
 #include "nvim/memory.h"
 #include "nvim/message.h"
-#include "nvim/move.h"
 #include "nvim/pos_defs.h"
 #include "nvim/sign.h"
 #include "nvim/sign_defs.h"
@@ -126,7 +124,7 @@ static void buf_set_sign(buf_T *buf, uint32_t *id, char *group, int prio, linenr
 
   DecorInline decor = { .ext = true, .data.ext = { .vt = NULL, .sh_idx = decor_put_sh(sign) } };
   extmark_set(buf, ns, id, MIN(buf->b_ml.ml_line_count, lnum) - 1, 0, -1, -1,
-              decor, decor_flags, true, false, true, true, false, NULL);
+              decor, decor_flags, true, false, true, true, NULL);
 }
 
 /// For an existing, placed sign with "id", modify the sign, group or priority.
@@ -269,7 +267,7 @@ static void sign_list_placed(buf_T *rbuf, char *group)
   while (buf != NULL && !got_int) {
     if (buf_has_signs(buf)) {
       vim_snprintf(lbuf, MSG_BUF_LEN, _("Signs for %s:"), buf->b_fname);
-      msg_puts_attr(lbuf, HL_ATTR(HLF_D));
+      msg_puts_hl(lbuf, HLF_D, false);
       msg_putchar('\n');
     }
 
@@ -291,8 +289,8 @@ static void sign_list_placed(buf_T *rbuf, char *group)
         qsort((void *)&kv_A(signs, 0), kv_size(signs), sizeof(MTKey), sign_row_cmp);
 
         for (size_t i = 0; i < kv_size(signs); i++) {
-          namebuf[0] = '\0';
-          groupbuf[0] = '\0';
+          namebuf[0] = NUL;
+          groupbuf[0] = NUL;
           MTKey mark = kv_A(signs, i);
 
           DecorSignHighlight *sh = decor_find_sign(mt_decor(mark));
@@ -375,7 +373,7 @@ int init_sign_text(sign_T *sp, schar_T *sign_text, char *text)
     if (!vim_isprintc(c)) {
       break;
     }
-    int width = utf_char2cells(c);
+    int width = utf_ptr2cells(s);
     if (width == 2) {
       sign_text[cells + 1] = 0;
     }
@@ -400,7 +398,7 @@ int init_sign_text(sign_T *sp, schar_T *sign_text, char *text)
 
 /// Define a new sign or update an existing sign
 static int sign_define_by_name(char *name, char *icon, char *text, char *linehl, char *texthl,
-                               char *culhl, char *numhl)
+                               char *culhl, char *numhl, int prio)
 {
   cstr_t *key;
   bool new_sign = false;
@@ -424,6 +422,8 @@ static int sign_define_by_name(char *name, char *icon, char *text, char *linehl,
     return FAIL;
   }
 
+  (*sp)->sn_priority = prio;
+
   char *arg[] = { linehl, texthl, culhl, numhl };
   int *hl[] = { &(*sp)->sn_line_hl, &(*sp)->sn_text_hl, &(*sp)->sn_cul_hl, &(*sp)->sn_num_hl };
   for (int i = 0; i < 4; i++) {
@@ -478,14 +478,19 @@ static void sign_list_defined(sign_T *sp)
   smsg(0, "sign %s", sp->sn_name);
   if (sp->sn_icon != NULL) {
     msg_puts(" icon=");
-    msg_outtrans(sp->sn_icon, 0);
+    msg_outtrans(sp->sn_icon, 0, false);
     msg_puts(_(" (not supported)"));
   }
   if (sp->sn_text[0]) {
     msg_puts(" text=");
     char buf[SIGN_WIDTH * MAX_SCHAR_SIZE];
     describe_sign_text(buf, sp->sn_text);
-    msg_outtrans(buf, 0);
+    msg_outtrans(buf, 0, false);
+  }
+  if (sp->sn_priority > 0) {
+    char lbuf[MSG_BUF_LEN];
+    vim_snprintf(lbuf, MSG_BUF_LEN, " priority=%d", sp->sn_priority);
+    msg_puts(lbuf);
   }
   static char *arg[] = { " linehl=", " texthl=", " culhl=", " numhl=" };
   int hl[] = { sp->sn_line_hl, sp->sn_text_hl, sp->sn_cul_hl, sp->sn_num_hl };
@@ -513,7 +518,7 @@ static void sign_list_by_name(char *name)
 static int sign_place(uint32_t *id, char *group, char *name, buf_T *buf, linenr_T lnum, int prio)
 {
   // Check for reserved character '*' in group name
-  if (group != NULL && (*group == '*' || *group == '\0')) {
+  if (group != NULL && (*group == '*' || *group == NUL)) {
     return FAIL;
   }
 
@@ -523,6 +528,11 @@ static int sign_place(uint32_t *id, char *group, char *name, buf_T *buf, linenr_
     return FAIL;
   }
 
+  // Use the default priority value for this sign.
+  if (prio == -1) {
+    prio = (sp->sn_priority != -1) ? sp->sn_priority : SIGN_DEF_PRIO;
+  }
+
   if (lnum > 0) {
     // ":sign place {id} line={lnum} name={name} file={fname}": place a sign
     buf_set_sign(buf, id, group, prio, lnum, sp);
@@ -617,6 +627,7 @@ static void sign_define_cmd(char *name, char *cmdline)
   char *texthl = NULL;
   char *culhl = NULL;
   char *numhl = NULL;
+  int prio = -1;
 
   // set values for a defined sign.
   while (true) {
@@ -637,6 +648,8 @@ static void sign_define_cmd(char *name, char *cmdline)
       culhl = arg + 6;
     } else if (strncmp(arg, "numhl=", 6) == 0) {
       numhl = arg + 6;
+    } else if (strncmp(arg, "priority=", 9) == 0) {
+      prio = atoi(arg + 9);
     } else {
       semsg(_(e_invarg2), arg);
       return;
@@ -647,7 +660,7 @@ static void sign_define_cmd(char *name, char *cmdline)
     *cmdline++ = NUL;
   }
 
-  sign_define_by_name(name, icon, text, linehl, texthl, culhl, numhl);
+  sign_define_by_name(name, icon, text, linehl, texthl, culhl, numhl, prio);
 }
 
 /// ":sign place" command
@@ -664,14 +677,14 @@ static void sign_place_cmd(buf_T *buf, linenr_T lnum, char *name, int id, char *
     //   :sign place
     //   :sign place group={group}
     //   :sign place group=*
-    if (lnum >= 0 || name != NULL || (group != NULL && *group == '\0')) {
+    if (lnum >= 0 || name != NULL || (group != NULL && *group == NUL)) {
       emsg(_(e_invarg));
     } else {
       sign_list_placed(buf, group);
     }
   } else {
     // Place a new sign
-    if (name == NULL || buf == NULL || (group != NULL && *group == '\0')) {
+    if (name == NULL || buf == NULL || (group != NULL && *group == NUL)) {
       emsg(_(e_invarg));
       return;
     }
@@ -683,7 +696,7 @@ static void sign_place_cmd(buf_T *buf, linenr_T lnum, char *name, int id, char *
 /// ":sign unplace" command
 static void sign_unplace_cmd(buf_T *buf, linenr_T lnum, const char *name, int id, char *group)
 {
-  if (lnum >= 0 || name != NULL || (group != NULL && *group == '\0')) {
+  if (lnum >= 0 || name != NULL || (group != NULL && *group == NUL)) {
     emsg(_(e_invarg));
     return;
   }
@@ -710,7 +723,7 @@ static void sign_jump_cmd(buf_T *buf, linenr_T lnum, const char *name, int id, c
     return;
   }
 
-  if (buf == NULL || (group != NULL && *group == '\0') || lnum >= 0 || name != NULL) {
+  if (buf == NULL || (group != NULL && *group == NUL) || lnum >= 0 || name != NULL) {
     // File or buffer is not specified or an empty group is used
     // or a line number or a sign name is specified.
     emsg(_(e_invarg));
@@ -862,7 +875,7 @@ void ex_sign(exarg_T *eap)
     linenr_T lnum = -1;
     char *name = NULL;
     char *group = NULL;
-    int prio = SIGN_DEF_PRIO;
+    int prio = -1;
     buf_T *buf = NULL;
 
     // Parse command line arguments
@@ -895,6 +908,9 @@ static dict_T *sign_get_info_dict(sign_T *sp)
     describe_sign_text(buf, sp->sn_text);
     tv_dict_add_str(d, S_LEN("text"), buf);
   }
+  if (sp->sn_priority > 0) {
+    tv_dict_add_nr(d, S_LEN("priority"), sp->sn_priority);
+  }
   static char *arg[] = { "linehl", "texthl", "culhl", "numhl" };
   int hl[] = { sp->sn_line_hl, sp->sn_text_hl, sp->sn_cul_hl, sp->sn_num_hl };
   for (int i = 0; i < 4; i++) {
@@ -1059,7 +1075,8 @@ char *get_sign_name(expand_T *xp, int idx)
   case EXP_SUBCMD:
     return cmds[idx];
   case EXP_DEFINE: {
-    char *define_arg[] = { "culhl=", "icon=", "linehl=", "numhl=", "text=", "texthl=", NULL };
+    char *define_arg[] = { "culhl=", "icon=", "linehl=", "numhl=", "text=", "texthl=",
+                           "priority=", NULL };
     return define_arg[idx];
   }
   case EXP_PLACE: {
@@ -1215,6 +1232,7 @@ static int sign_define_from_dict(char *name, dict_T *dict)
   char *texthl = NULL;
   char *culhl = NULL;
   char *numhl = NULL;
+  int prio = -1;
 
   if (dict != NULL) {
     icon = tv_dict_get_string(dict, "icon", false);
@@ -1223,9 +1241,10 @@ static int sign_define_from_dict(char *name, dict_T *dict)
     texthl = tv_dict_get_string(dict, "texthl", false);
     culhl = tv_dict_get_string(dict, "culhl", false);
     numhl = tv_dict_get_string(dict, "numhl", false);
+    prio = (int)tv_dict_get_number_def(dict, "priority", -1);
   }
 
-  return sign_define_by_name(name, icon, text, linehl, texthl, culhl, numhl) - 1;
+  return sign_define_by_name(name, icon, text, linehl, texthl, culhl, numhl, prio) - 1;
 }
 
 /// Define multiple signs using attributes from list 'l' and store the return
@@ -1331,7 +1350,7 @@ void f_sign_getplaced(typval_T *argvars, typval_T *rettv, EvalFuncData fptr)
         if (group == NULL) {
           return;
         }
-        if (*group == '\0') {  // empty string means global group
+        if (*group == NUL) {  // empty string means global group
           group = NULL;
         }
       }
@@ -1457,7 +1476,7 @@ static int sign_place_from_dict(typval_T *id_tv, typval_T *group_tv, typval_T *n
     }
   }
 
-  int prio = SIGN_DEF_PRIO;
+  int prio = -1;
   di = tv_dict_find(dict, "priority", -1);
   if (di != NULL) {
     prio = (int)tv_get_number_chk(&di->di_tv, &notanum);
