@@ -1,15 +1,15 @@
 #pragma once
 
 #include <stdbool.h>
+#include <stddef.h>
 #include <stdint.h>
 #include <uv.h>
 
 #include "nvim/event/defs.h"
-#include "nvim/rbuffer_defs.h"
 #include "nvim/tui/input_defs.h"  // IWYU pragma: keep
+#include "nvim/tui/termkey/termkey_defs.h"
 #include "nvim/tui/tui_defs.h"
 #include "nvim/types_defs.h"
-#include "termkey/termkey.h"
 
 typedef enum {
   kKeyEncodingLegacy,  ///< Legacy key encoding
@@ -17,14 +17,18 @@ typedef enum {
   kKeyEncodingXterm,   ///< Xterm's modifyOtherKeys encoding (XTMODKEYS)
 } KeyEncoding;
 
+typedef struct {
+  void (*primary_device_attr)(TUIData *tui);
+} TermInputCallbacks;
+
+#define KEY_BUFFER_SIZE 0x1000
 typedef struct {
   int in_fd;
   // Phases: -1=all 0=disabled 1=first-chunk 2=continue 3=last-chunk
   int8_t paste;
   bool ttimeout;
 
-  bool waiting_for_kkp_response;  ///< True if we are expecting to receive a response to a query for
-                                  ///< Kitty keyboard protocol support
+  TermInputCallbacks callbacks;
 
   KeyEncoding key_encoding;       ///< The key encoding used by the terminal emulator
 
@@ -32,18 +36,14 @@ typedef struct {
   TermKey *tk;
   TermKey_Terminfo_Getstr_Hook *tk_ti_hook_fn;  ///< libtermkey terminfo hook
   uv_timer_t timer_handle;
+  uv_timer_t bg_query_timer;  ///< timer used to batch background color queries
   Loop *loop;
-  Stream read_stream;
-  RBuffer *key_buffer;
+  RStream read_stream;
   TUIData *tui_data;
+  char key_buffer[KEY_BUFFER_SIZE];
+  size_t key_buffer_len;
 } TermInput;
 
-typedef enum {
-  kIncomplete = -1,
-  kNotApplicable = 0,
-  kComplete = 1,
-} HandleState;
-
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "tui/input.h.generated.h"
 #endif
