@@ -18,7 +18,6 @@
 #include "nvim/garray.h"
 #include "nvim/gettext_defs.h"
 #include "nvim/globals.h"
-#include "nvim/highlight.h"
 #include "nvim/highlight_defs.h"
 #include "nvim/keycodes.h"
 #include "nvim/lua/executor.h"
@@ -26,6 +25,7 @@
 #include "nvim/mapping.h"
 #include "nvim/mbyte.h"
 #include "nvim/memory.h"
+#include "nvim/memory_defs.h"
 #include "nvim/menu.h"
 #include "nvim/message.h"
 #include "nvim/option_vars.h"
@@ -91,6 +91,7 @@ static const char *command_complete[] = {
   [EXPAND_PACKADD] = "packadd",
   [EXPAND_RUNTIME] = "runtime",
   [EXPAND_SHELLCMD] = "shellcmd",
+  [EXPAND_SHELLCMDLINE] = "shellcmdline",
   [EXPAND_SIGN] = "sign",
   [EXPAND_TAGS] = "tag",
   [EXPAND_TAGS_LISTFILES] = "tag_listfiles",
@@ -98,6 +99,7 @@ static const char *command_complete[] = {
   [EXPAND_USER_VARS] = "var",
   [EXPAND_BREAKPOINT] = "breakpoint",
   [EXPAND_SCRIPTNAMES] = "scriptnames",
+  [EXPAND_DIRS_IN_CDPATH] = "dir_in_path",
 };
 
 /// List of names of address types.  Must be alphabetical for completion.
@@ -284,8 +286,7 @@ const char *set_context_in_user_cmdarg(const char *cmd FUNC_ATTR_UNUSED, const c
   }
 
   if (argt & EX_XFILE) {
-    // EX_XFILE: file names are handled above.
-    xp->xp_context = context;
+    // EX_XFILE: file names are handled before this call.
     return NULL;
   }
 
@@ -463,8 +464,7 @@ static void uc_list(char *name, size_t name_len)
 
       // Put out the title first time
       if (!found) {
-        msg_puts_title(_("\n    Name              Args Address "
-                         "Complete    Definition"));
+        msg_puts_title(_("\n    Name              Args Address Complete    Definition"));
       }
       found = true;
       msg_putchar('\n');
@@ -494,7 +494,7 @@ static void uc_list(char *name, size_t name_len)
         msg_putchar(' ');
       }
 
-      msg_outtrans(cmd->uc_name, HL_ATTR(HLF_D));
+      msg_outtrans(cmd->uc_name, HLF_D, false);
       len = strlen(cmd->uc_name) + 4;
 
       do {
@@ -580,12 +580,12 @@ static void uc_list(char *name, size_t name_len)
         IObuff[len++] = ' ';
       } while ((int64_t)len < 25 - over);
 
-      IObuff[len] = '\0';
-      msg_outtrans(IObuff, 0);
+      IObuff[len] = NUL;
+      msg_outtrans(IObuff, 0, false);
 
       if (cmd->uc_luaref != LUA_NOREF) {
         char *fn = nlua_funcref_str(cmd->uc_luaref, NULL);
-        msg_puts_attr(fn, HL_ATTR(HLF_8));
+        msg_puts_hl(fn, HLF_8, false);
         xfree(fn);
         // put the description on a new line
         if (*cmd->uc_rep != NUL) {
@@ -674,7 +674,8 @@ int parse_compl_arg(const char *value, int vallen, int *complp, uint32_t *argt,
       *complp = i;
       if (i == EXPAND_BUFFERS) {
         *argt |= EX_BUFNAME;
-      } else if (i == EXPAND_DIRECTORIES || i == EXPAND_FILES) {
+      } else if (i == EXPAND_DIRECTORIES || i == EXPAND_FILES
+                 || i == EXPAND_SHELLCMDLINE) {
         *argt |= EX_XFILE;
       }
       break;
@@ -804,9 +805,7 @@ invalid_count:
         }
       }
 
-      if (*def < 0) {
-        *def = 0;
-      }
+      *def = MAX(*def, 0);
     } else if (STRNICMP(attr, "complete", attrlen) == 0) {
       if (val == NULL) {
         semsg(_(e_argument_required_for_str), "-complete");
@@ -831,7 +830,7 @@ invalid_count:
       }
     } else {
       char ch = attr[len];
-      attr[len] = '\0';
+      attr[len] = NUL;
       semsg(_("E181: Invalid attribute: %s"), attr);
       attr[len] = ch;
       return FAIL;
@@ -1274,9 +1273,9 @@ static size_t add_cmd_modifier(char *buf, char *mod_str, bool *multi_mods)
 
   if (buf != NULL) {
     if (*multi_mods) {
-      STRCAT(buf, " ");
+      strcat(buf, " ");
     }
-    STRCAT(buf, mod_str);
+    strcat(buf, mod_str);
   }
 
   *multi_mods = true;
@@ -1364,7 +1363,7 @@ size_t uc_mods(char *buf, const cmdmod_T *cmod, bool quote)
     if (quote) {
       *buf++ = '"';
     }
-    *buf = '\0';
+    *buf = NUL;
   }
 
   // the modifiers that are simple flags
@@ -1744,14 +1743,14 @@ int do_ucmd(exarg_T *eap, bool preview)
 /// @param buf  Buffer to inspect, or NULL to get global commands.
 ///
 /// @return Map of maps describing commands
-Dictionary commands_array(buf_T *buf, Arena *arena)
+Dict commands_array(buf_T *buf, Arena *arena)
 {
   garray_T *gap = (buf == NULL) ? &ucmds : &buf->b_ucmds;
 
-  Dictionary rv = arena_dict(arena, (size_t)gap->ga_len);
+  Dict rv = arena_dict(arena, (size_t)gap->ga_len);
   for (int i = 0; i < gap->ga_len; i++) {
     char arg[2] = { 0, 0 };
-    Dictionary d = arena_dict(arena, 14);
+    Dict d = arena_dict(arena, 14);
     ucmd_T *cmd = USER_CMD_GA(gap, i);
 
     PUT_C(d, "name", CSTR_AS_OBJ(cmd->uc_name));
@@ -1815,7 +1814,7 @@ Dictionary commands_array(buf_T *buf, Arena *arena)
     }
     PUT_C(d, "addr", obj);
 
-    PUT_C(rv, cmd->uc_name, DICTIONARY_OBJ(d));
+    PUT_C(rv, cmd->uc_name, DICT_OBJ(d));
   }
   return rv;
 }
