@@ -1,12 +1,15 @@
-#ifndef NVIM_VERSION_H
-#define NVIM_VERSION_H
+#pragma once
 
-#include "nvim/ex_cmds_defs.h"
-#include "nvim/macros.h"
+#include "nvim/api/private/defs.h"  // IWYU pragma: keep
+#include "nvim/ex_cmds_defs.h"  // IWYU pragma: keep
+#include "nvim/macros_defs.h"
 
 // defined in version.c
 extern char *Version;
 extern char *longVersion;
+#ifndef NDEBUG
+extern char *version_cflags;
+#endif
 
 //
 // Vim version number, name, etc. Patchlevel is defined in version.c.
@@ -29,4 +32,3 @@ extern char *longVersion;
 #ifdef INCLUDE_GENERATED_DECLARATIONS
 # include "version.h.generated.h"
 #endif
-#endif  // NVIM_VERSION_H
