@@ -42,59 +42,62 @@ describe('named marks', function()
   it('errors when set out of range with :mark', function()
     command('edit ' .. file1)
     local err = pcall_err(n.exec_capture, '1000mark x')
-    eq('nvim_exec2(): Vim(mark):E16: Invalid range: 1000mark x', err)
+    eq('nvim_exec2(), line 1: Vim(mark):E16: Invalid range: 1000mark x', err)
   end)
 
   it('errors when set out of range with :k', function()
     command('edit ' .. file1)
     local err = pcall_err(n.exec_capture, '1000kx')
-    eq('nvim_exec2(): Vim(k):E16: Invalid range: 1000kx', err)
+    eq('nvim_exec2(), line 1: Vim(k):E16: Invalid range: 1000kx', err)
   end)
 
   it('errors on unknown mark name with :mark', function()
     command('edit ' .. file1)
     local err = pcall_err(n.exec_capture, 'mark #')
-    eq('nvim_exec2(): Vim(mark):E191: Argument must be a letter or forward/backward quote', err)
+    eq(
+      'nvim_exec2(), line 1: Vim(mark):E191: Argument must be a letter or forward/backward quote',
+      err
+    )
   end)
 
   it("errors on unknown mark name with '", function()
     command('edit ' .. file1)
     local err = pcall_err(n.exec_capture, "normal! '#")
-    eq('nvim_exec2(): Vim(normal):E78: Unknown mark', err)
+    eq('nvim_exec2(), line 1: Vim(normal):E78: Unknown mark', err)
   end)
 
   it('errors on unknown mark name with `', function()
     command('edit ' .. file1)
     local err = pcall_err(n.exec_capture, 'normal! `#')
-    eq('nvim_exec2(): Vim(normal):E78: Unknown mark', err)
+    eq('nvim_exec2(), line 1: Vim(normal):E78: Unknown mark', err)
   end)
 
   it("errors when moving to a mark that is not set with '", function()
     command('edit ' .. file1)
     local err = pcall_err(n.exec_capture, "normal! 'z")
-    eq('nvim_exec2(): Vim(normal):E20: Mark not set', err)
+    eq('nvim_exec2(), line 1: Vim(normal):E20: Mark not set', err)
     err = pcall_err(n.exec_capture, "normal! '.")
-    eq('nvim_exec2(): Vim(normal):E20: Mark not set', err)
+    eq('nvim_exec2(), line 1: Vim(normal):E20: Mark not set', err)
   end)
 
   it('errors when moving to a mark that is not set with `', function()
     command('edit ' .. file1)
     local err = pcall_err(n.exec_capture, 'normal! `z')
-    eq('nvim_exec2(): Vim(normal):E20: Mark not set', err)
+    eq('nvim_exec2(), line 1: Vim(normal):E20: Mark not set', err)
     err = pcall_err(n.exec_capture, 'normal! `>')
-    eq('nvim_exec2(): Vim(normal):E20: Mark not set', err)
+    eq('nvim_exec2(), line 1: Vim(normal):E20: Mark not set', err)
   end)
 
   it("errors when moving to a global mark that is not set with '", function()
     command('edit ' .. file1)
     local err = pcall_err(n.exec_capture, "normal! 'Z")
-    eq('nvim_exec2(): Vim(normal):E20: Mark not set', err)
+    eq('nvim_exec2(), line 1: Vim(normal):E20: Mark not set', err)
   end)
 
   it('errors when moving to a global mark that is not set with `', function()
     command('edit ' .. file1)
     local err = pcall_err(n.exec_capture, 'normal! `Z')
-    eq('nvim_exec2(): Vim(normal):E20: Mark not set', err)
+    eq('nvim_exec2(), line 1: Vim(normal):E20: Mark not set', err)
   end)
 
   it("can move to them using '", function()
@@ -169,7 +172,7 @@ describe('named marks', function()
     command('next')
     command('bw! ' .. file1)
     local err = pcall_err(n.exec_capture, "normal! 'A")
-    eq('nvim_exec2(): Vim(normal):E92: Buffer 1 not found', err)
+    eq('nvim_exec2(), line 1: Vim(normal):E92: Buffer 1 not found', err)
     os.remove(file1)
   end)
 
@@ -348,7 +351,6 @@ describe('named marks view', function()
 
   it('is restored in normal mode but not op-pending mode', function()
     local screen = Screen.new(5, 8)
-    screen:attach()
     command('edit ' .. file1)
     feed('<C-e>jWma')
     feed("G'a")
@@ -390,7 +392,6 @@ describe('named marks view', function()
 
   it('is restored across files', function()
     local screen = Screen.new(5, 5)
-    screen:attach()
     command('args ' .. file1 .. ' ' .. file2)
     feed('<C-e>mA')
     local mark_view = [[
@@ -415,7 +416,6 @@ describe('named marks view', function()
 
   it("fallback to standard behavior when view can't be recovered", function()
     local screen = Screen.new(10, 10)
-    screen:attach()
     command('edit ' .. file1)
     feed('7GzbmaG') -- Seven lines from the top
     command('new') -- Screen size for window is now half the height can't be restored
@@ -434,7 +434,6 @@ describe('named marks view', function()
 
   it('fallback to standard behavior when mark is loaded from shada', function()
     local screen = Screen.new(10, 6)
-    screen:attach()
     command('edit ' .. file1)
     feed('G')
     feed('mA')
