@@ -9,6 +9,7 @@ local feed = n.feed
 local fn = n.fn
 local command = n.command
 local eq = t.eq
+local api = n.api
 
 describe('Normal mode', function()
   before_each(clear)
@@ -25,11 +26,10 @@ describe('Normal mode', function()
 
   it('&showcmd does not crash with :startinsert #28419', function()
     local screen = Screen.new(60, 17)
-    screen:attach()
-    fn.termopen(
-      { n.nvim_prog, '--clean', '--cmd', 'startinsert' },
-      { env = { VIMRUNTIME = os.getenv('VIMRUNTIME') } }
-    )
+    fn.jobstart({ n.nvim_prog, '--clean', '--cmd', 'startinsert' }, {
+      term = true,
+      env = { VIMRUNTIME = os.getenv('VIMRUNTIME') },
+    })
     screen:expect({
       grid = [[
         ^                                                            |
@@ -41,4 +41,22 @@ describe('Normal mode', function()
       attr_ids = {},
     })
   end)
+
+  it('replacing with ZWJ emoji sequences', function()
+    local screen = Screen.new(30, 8)
+    api.nvim_buf_set_lines(0, 0, -1, true, { 'abcdefg' })
+    feed('05r🧑‍🌾') -- ZWJ
+    screen:expect([[
+      🧑‍🌾🧑‍🌾🧑‍🌾🧑‍🌾^🧑‍🌾fg                  |
+      {1:~                             }|*6
+                                    |
+    ]])
+
+    feed('2r🏳️‍⚧️') -- ZWJ and variant selectors
+    screen:expect([[
+      🧑‍🌾🧑‍🌾🧑‍🌾🧑‍🌾🏳️‍⚧️^🏳️‍⚧️g                 |
+      {1:~                             }|*6
+                                    |
+    ]])
+  end)
 end)
