@@ -1,8 +1,9 @@
 -- Tests for [ CTRL-I with a count and CTRL-W CTRL-I with a count
 
-local helpers = require('test.functional.helpers')(after_each)
-local clear, feed, insert = helpers.clear, helpers.feed, helpers.insert
-local feed_command, expect = helpers.feed_command, helpers.expect
+local n = require('test.functional.testnvim')()
+
+local clear, feed, insert = n.clear, n.feed, n.insert
+local feed_command, expect = n.feed_command, n.expect
 
 describe('CTRL-W CTRL-I', function()
   setup(clear)
