@@ -18,7 +18,6 @@ describe('prompt buffer', function()
   before_each(function()
     clear()
     screen = Screen.new(25, 10)
-    screen:attach()
     command('set laststatus=0 nohidden')
   end)
 
@@ -68,10 +67,6 @@ describe('prompt buffer', function()
     ]])
   end
 
-  after_each(function()
-    screen:detach()
-  end)
-
   -- oldtest: Test_prompt_basic()
   it('works', function()
     source_script()
