@@ -38,7 +38,6 @@ describe('Diff mode screen with 3 diffs open', function()
     feed(':windo diffthis<cr>')
 
     screen = Screen.new(100, 16)
-    screen:attach()
     feed('<c-w>=')
     feed(':windo set nu!<cr>')
   end)
@@ -96,8 +95,7 @@ describe('Diff mode screen with 3 diffs open', function()
       {7:  }{8:  9 }{4:    BBB                    }│{7:  }{8:  9 }{4:    BBB                   }│{7:  }{8:    }{23:---------------------------}|
       {7:  }{8: 10 }{4:    BBB                    }│{7:  }{8: 10 }{4:    BBB                   }│{7:  }{8:    }{23:---------------------------}|
       {7:  }{8: 11 }{4:>>>>>>> branch1            }│{7:  }{8: 11 }{4:>>>>>>> branch1           }│{7:  }{8:    }{23:---------------------------}|
-      {7:  }{8: 12 }                           │{7:  }{8: 12 }                          │{7:  }{8:  6 }                           |
-      {1:~                                }│{1:~                               }│{1:~                                }|*2
+      {1:~                                }│{1:~                               }│{1:~                                }|*3
       {3:<-functional-diff-screen-1.3 [+]  }{2:<est-functional-diff-screen-1.2  Xtest-functional-diff-screen-1   }|
       :2,6diffget screen-1.2                                                                              |
       ]])
@@ -115,8 +113,7 @@ describe('Diff mode screen with 3 diffs open', function()
       {7:  }{8:  4 }{4:    }{27:BBB}{4:                    }│{7:  }{8:  6 }{4:    }{27:BBB}{4:                   }│{7:  }{8:  4 }{4:    }{27:AAA}{4:                    }|
       {7:  }{8:  5 }{4:    }{27:BBB}{4:                    }│{7:  }{8:  7 }{4:    }{27:BBB}{4:                   }│{7:  }{8:  5 }{4:    }{27:AAA}{4:                    }|
       {7:  }{8:    }{23:---------------------------}│{7:  }{8:  8 }{22:>>>>>>> branch1           }│{7:  }{8:    }{23:---------------------------}|
-      {7:  }{8:  6 }                           │{7:  }{8:  9 }                          │{7:  }{8:  6 }                           |
-      {1:~                                }│{1:~                               }│{1:~                                }|*5
+      {1:~                                }│{1:~                               }│{1:~                                }|*6
       {2:<test-functional-diff-screen-1.3  }{3:<functional-diff-screen-1.2 [+]  }{2:Xtest-functional-diff-screen-1   }|
       :5,7diffget screen-1.3                                                                              |
       ]])
@@ -137,8 +134,7 @@ describe('Diff mode screen with 3 diffs open', function()
       {7:  }{8:  4 }    BBB                    │{7:  }{8:  9 }    BBB                   │{7:  }{8:  8 }    BBB                    |
       {7:  }{8:  5 }    BBB                    │{7:  }{8: 10 }    BBB                   │{7:  }{8:  9 }    BBB                    |
       {7:  }{8:    }{23:---------------------------}│{7:  }{8: 11 }{4:>>>>>>> branch1           }│{7:  }{8: 10 }{4:>>>>>>> branch1            }|
-      {7:  }{8:  6 }                           │{7:  }{8: 12 }                          │{7:  }{8: 11 }                           |
-      {1:~                                }│{1:~                               }│{1:~                                }|*2
+      {1:~                                }│{1:~                               }│{1:~                                }|*3
       {2:<test-functional-diff-screen-1.3  <est-functional-diff-screen-1.2  }{3:<st-functional-diff-screen-1 [+] }|
       :5,6diffget screen-1.2                                                                              |
       ]])
@@ -159,8 +155,7 @@ describe('Diff mode screen with 3 diffs open', function()
       {7:  }{8:  4 }{4:    BBB                    }│{7:  }{8:  9 }{4:    BBB                   }│{7:  }{8:    }{23:---------------------------}|
       {7:  }{8:  5 }    BBB                    │{7:  }{8: 10 }    BBB                   │{7:  }{8:  7 }    BBB                    |
       {7:  }{8:    }{23:---------------------------}│{7:  }{8: 11 }{22:>>>>>>> branch1           }│{7:  }{8:    }{23:---------------------------}|
-      {7:  }{8:  6 }                           │{7:  }{8: 12 }                          │{7:  }{8:  8 }                           |
-      {1:~                                }│{1:~                               }│{1:~                                }|*2
+      {1:~                                }│{1:~                               }│{1:~                                }|*3
       {2:<test-functional-diff-screen-1.3  }{3:<est-functional-diff-screen-1.2  }{2:<st-functional-diff-screen-1 [+] }|
       :6,8diffput screen-1                                                                                |
       ]])
@@ -180,8 +175,7 @@ describe('Diff mode screen with 3 diffs open', function()
       {7:  }{8:  4 }    BBB                    │{7:  }{8:  9 }    BBB                   │{7:  }{8:  8 }    BBB                    |
       {7:  }{8:  5 }    BBB                    │{7:  }{8: 10 }    BBB                   │{7:  }{8:  9 }    BBB                    |
       {7:  }{8:    }{23:---------------------------}│{7:  }{8: 11 }{4:>>>>>>> branch1           }│{7:  }{8: 10 }{4:>>>>>>> branch1            }|
-      {7:  }{8:  6 }                           │{7:  }{8: 12 }                          │{7:  }{8: 11 }                           |
-      {1:~                                }│{1:~                               }│{1:~                                }|*2
+      {1:~                                }│{1:~                               }│{1:~                                }|*3
       {2:<test-functional-diff-screen-1.3  }{3:<est-functional-diff-screen-1.2  }{2:<st-functional-diff-screen-1 [+] }|
       :6,11diffput screen-1                                                                               |
       ]])
@@ -217,7 +211,6 @@ describe('Diff mode screen with 2 diffs open', function()
     feed(':windo diffthis<cr>')
 
     screen = Screen.new(100, 20)
-    screen:attach()
     feed('<c-w>=')
     feed(':windo set nu!<cr>')
   end)
@@ -278,8 +271,7 @@ something
       {7:  }{8: 14 }common line                                │{7:  }{8: 15 }common line                                 |
       {7:  }{8:    }{23:-------------------------------------------}│{7:  }{8: 16 }{22:DEF                                         }|
       {7:  }{8: 15 }something                                  │{7:  }{8: 17 }something                                   |
-      {7:  }{8: 16 }                                           │{7:  }{8: 18 }                                            |
-      {1:~                                                }│{1:~                                                 }|*6
+      {1:~                                                }│{1:~                                                 }|*7
       {3:Xtest-functional-diff-screen-1.2 [+]              }{2:Xtest-functional-diff-screen-1                    }|
       :5,9diffget                                                                                         |
       ]])
@@ -302,8 +294,7 @@ something
       {7:  }{8: 11 }common line                                │{7:  }{8: 12 }common line                                 |
       {7:  }{8:    }{23:-------------------------------------------}│{7:  }{8: 13 }{22:DEF                                         }|
       {7:  }{8: 12 }something                                  │{7:  }{8: 14 }something                                   |
-      {7:  }{8: 13 }                                           │{7:  }{8: 15 }                                            |
-      {1:~                                                }│{1:~                                                 }|*3
+      {1:~                                                }│{1:~                                                 }|*4
       {2:Xtest-functional-diff-screen-1.2                  }{3:Xtest-functional-diff-screen-1 [+]                }|
       :5,10diffget                                                                                        |
       ]])
@@ -324,8 +315,7 @@ something
       {7:  }{8: 10 }common line                                │{7:  }{8: 10 }common line                                 |
       {7:  }{8: 11 }common line                                │{7:  }{8: 11 }common line                                 |
       {7:  }{8: 12 }something                                  │{7:  }{8: 12 }something                                   |
-      {7:  }{8: 13 }                                           │{7:  }{8: 13 }                                            |
-      {1:~                                                }│{1:~                                                 }|*5
+      {1:~                                                }│{1:~                                                 }|*6
       {2:Xtest-functional-diff-screen-1.2                  }{3:Xtest-functional-diff-screen-1 [+]                }|
       :4,17diffget                                                                                        |
       ]])
@@ -351,7 +341,7 @@ something
       {7:  }{8: 15 }common line                                │{7:  }{8: 15 }common line                                 |
       {7:  }{8: 16 }DEF                                        │{7:  }{8: 16 }DEF                                         |
       {7:  }{8: 17 }something                                  │{7:  }{8: 17 }something                                   |
-      {7:  }{8: 18 }                                           │{7:  }{8: 18 }                                            |
+      {1:~                                                }│{1:~                                                 }|
       {3:Xtest-functional-diff-screen-1.2 [+]              }{2:Xtest-functional-diff-screen-1                    }|
       :4,12diffget                                                                                        |
       ]])
@@ -378,7 +368,7 @@ something
       {7:  }{8: 11 }common line                                │{7:  }{8: 15 }common line                                 |
       {7:  }{8:    }{23:-------------------------------------------}│{7:  }{8: 16 }{22:DEF                                         }|
       {7:  }{8: 12 }something                                  │{7:  }{8: 17 }something                                   |
-      {7:  }{8: 13 }                                           │{7:  }{8: 18 }                                            |
+      {1:~                                                }│{1:~                                                 }|
       {3:Xtest-functional-diff-screen-1.2 [+]              }{2:Xtest-functional-diff-screen-1                    }|
       :e                                                                                                  |
       ]])
@@ -405,7 +395,7 @@ something
       {7:  }{8: 11 }common line                                │{7:  }{8: 15 }common line                                 |
       {7:  }{8:    }{23:-------------------------------------------}│{7:  }{8: 16 }{22:DEF                                         }|
       {7:  }{8: 12 }something                                  │{7:  }{8: 17 }something                                   |
-      {7:  }{8: 13 }                                           │{7:  }{8: 18 }                                            |
+      {1:~                                                }│{1:~                                                 }|
       {3:Xtest-functional-diff-screen-1.2 [+]              }{2:Xtest-functional-diff-screen-1                    }|
       :e                                                                                                  |
       ]])
@@ -432,7 +422,7 @@ something
       {7:  }{8: 11 }common line                                │{7:  }{8: 15 }common line                                 |
       {7:  }{8:    }{23:-------------------------------------------}│{7:  }{8: 16 }{22:DEF                                         }|
       {7:  }{8: 12 }something                                  │{7:  }{8: 17 }something                                   |
-      {7:  }{8: 13 }                                           │{7:  }{8: 18 }                                            |
+      {1:~                                                }│{1:~                                                 }|
       {3:Xtest-functional-diff-screen-1.2 [+]              }{2:Xtest-functional-diff-screen-1                    }|
       :e                                                                                                  |
       ]])
@@ -459,7 +449,7 @@ something
       {7:  }{8: 12 }^common line                                │{7:  }{8: 15 }common line                                 |
       {7:  }{8:    }{23:-------------------------------------------}│{7:  }{8: 16 }{22:DEF                                         }|
       {7:  }{8: 13 }something                                  │{7:  }{8: 17 }something                                   |
-      {7:  }{8: 14 }                                           │{7:  }{8: 18 }                                            |
+      {1:~                                                }│{1:~                                                 }|
       {3:Xtest-functional-diff-screen-1.2 [+]              }{2:Xtest-functional-diff-screen-1                    }|
       :e                                                                                                  |
       ]])
@@ -486,7 +476,7 @@ something
       {7:  }{8: 11 }common line                                │{7:  }{8: 15 }common line                                 |
       {7:  }{8: 12 }DEF                                        │{7:  }{8: 16 }DEF                                         |
       {7:  }{8: 13 }^something                                  │{7:  }{8: 17 }something                                   |
-      {7:  }{8: 14 }                                           │{7:  }{8: 18 }                                            |
+      {1:~                                                }│{1:~                                                 }|
       {3:Xtest-functional-diff-screen-1.2 [+]              }{2:Xtest-functional-diff-screen-1                    }|
       :e                                                                                                  |
       ]])
@@ -513,7 +503,7 @@ something
       {7:  }{8: 11 }common line                                │{7:  }{8: 15 }common line                                 |
       {7:  }{8:    }{23:-------------------------------------------}│{7:  }{8: 16 }{22:DEF                                         }|
       {7:  }{8: 12 }something                                  │{7:  }{8: 17 }something                                   |
-      {7:  }{8: 13 }                                           │{7:  }{8: 18 }                                            |
+      {1:~                                                }│{1:~                                                 }|
       {3:Xtest-functional-diff-screen-1.2                  }{2:Xtest-functional-diff-screen-1 [+]                }|
       :e                                                                                                  |
       ]])
@@ -540,7 +530,7 @@ something
       {7:  }{8: 11 }common line                                │{7:  }{8: 15 }common line                                 |
       {7:  }{8:    }{23:-------------------------------------------}│{7:  }{8: 16 }{22:DEF                                         }|
       {7:  }{8: 12 }something                                  │{7:  }{8: 17 }something                                   |
-      {7:  }{8: 13 }                                           │{7:  }{8: 18 }                                            |
+      {1:~                                                }│{1:~                                                 }|
       {3:Xtest-functional-diff-screen-1.2                  }{2:Xtest-functional-diff-screen-1 [+]                }|
       :e                                                                                                  |
       ]])
@@ -567,7 +557,7 @@ something
       {7:  }{8: 11 }common line                                │{7:  }{8: 15 }common line                                 |
       {7:  }{8:    }{23:-------------------------------------------}│{7:  }{8: 16 }{22:DEF                                         }|
       {7:  }{8: 12 }something                                  │{7:  }{8: 17 }something                                   |
-      {7:  }{8: 13 }                                           │{7:  }{8: 18 }                                            |
+      {1:~                                                }│{1:~                                                 }|
       {3:Xtest-functional-diff-screen-1.2                  }{2:Xtest-functional-diff-screen-1 [+]                }|
       :e                                                                                                  |
       ]])
@@ -593,7 +583,7 @@ something
       {7:  }{8: 11 }^common line                                │{7:  }{8: 14 }common line                                 |
       {7:  }{8:    }{23:-------------------------------------------}│{7:  }{8: 15 }{22:DEF                                         }|
       {7:  }{8: 12 }something                                  │{7:  }{8: 16 }something                                   |
-      {7:  }{8: 13 }                                           │{7:  }{8: 17 }                                            |
+      {1:~                                                }│{1:~                                                 }|
       {1:~                                                }│{1:~                                                 }|
       {3:Xtest-functional-diff-screen-1.2                  }{2:Xtest-functional-diff-screen-1 [+]                }|
       :e                                                                                                  |
@@ -620,7 +610,7 @@ something
       {7:  }{8:    }{23:-------------------------------------------}│{7:  }{8: 14 }{22:DEF                                         }|
       {7:  }{8: 11 }common line                                │{7:  }{8: 15 }common line                                 |
       {7:  }{8: 12 }^something                                  │{7:  }{8: 16 }something                                   |
-      {7:  }{8: 13 }                                           │{7:  }{8: 17 }                                            |
+      {1:~                                                }│{1:~                                                 }|
       {1:~                                                }│{1:~                                                 }|
       {3:Xtest-functional-diff-screen-1.2                  }{2:Xtest-functional-diff-screen-1 [+]                }|
       :e                                                                                                  |
@@ -648,7 +638,7 @@ something
       {7:  }{8: 14 }common line                                │{7:  }{8: 15 }common line                                 |
       {7:  }{8:    }{23:-------------------------------------------}│{7:  }{8: 16 }{22:DEF                                         }|
       {7:  }{8: 15 }something                                  │{7:  }{8: 17 }something                                   |
-      {7:  }{8: 16 }                                           │{7:  }{8: 18 }                                            |
+      {1:~                                                }│{1:~                                                 }|
       {2:Xtest-functional-diff-screen-1.2 [+]              }{3:Xtest-functional-diff-screen-1                    }|
       :e                                                                                                  |
       ]])
@@ -675,7 +665,7 @@ something
       {7:  }{8: 14 }common line                                │{7:  }{8: 15 }common line                                 |
       {7:  }{8:    }{23:-------------------------------------------}│{7:  }{8: 16 }{22:DEF                                         }|
       {7:  }{8: 15 }something                                  │{7:  }{8: 17 }something                                   |
-      {7:  }{8: 16 }                                           │{7:  }{8: 18 }                                            |
+      {1:~                                                }│{1:~                                                 }|
       {2:Xtest-functional-diff-screen-1.2 [+]              }{3:Xtest-functional-diff-screen-1                    }|
       :e                                                                                                  |
       ]])
@@ -702,7 +692,7 @@ something
       {7:  }{8: 11 }common line                                │{7:  }{8: 15 }common line                                 |
       {7:  }{8:    }{23:-------------------------------------------}│{7:  }{8: 16 }{22:DEF                                         }|
       {7:  }{8: 12 }something                                  │{7:  }{8: 17 }something                                   |
-      {7:  }{8: 13 }                                           │{7:  }{8: 18 }                                            |
+      {1:~                                                }│{1:~                                                 }|
       {2:Xtest-functional-diff-screen-1.2 [+]              }{3:Xtest-functional-diff-screen-1                    }|
       :e                                                                                                  |
       ]])
@@ -729,7 +719,7 @@ something
       {7:  }{8: 11 }common line                                │{7:  }{8: 15 }common line                                 |
       {7:  }{8: 12 }DEF                                        │{7:  }{8: 16 }DEF                                         |
       {7:  }{8: 13 }something                                  │{7:  }{8: 17 }^something                                   |
-      {7:  }{8: 14 }                                           │{7:  }{8: 18 }                                            |
+      {1:~                                                }│{1:~                                                 }|
       {2:Xtest-functional-diff-screen-1.2 [+]              }{3:Xtest-functional-diff-screen-1                    }|
       :e                                                                                                  |
       ]])
@@ -759,8 +749,7 @@ d
       {7:  }{8:  2 }{4:abc d                                      }│{7:  }{8:  1 }{27:// }{4:abc d                                    }|
       {7:  }{8:  3 }{4:d                                          }│{7:  }{8:  2 }{27:// }{4:d                                        }|
       {7:  }{8:    }{23:-------------------------------------------}│{7:  }{8:  3 }{22:// d                                        }|
-      {7:  }{8:  4 }                                           │{7:  }{8:  4 }                                            |
-      {1:~                                                }│{1:~                                                 }|*13
+      {1:~                                                }│{1:~                                                 }|*14
       {3:Xtest-functional-diff-screen-1.2                  }{2:Xtest-functional-diff-screen-1                    }|
       :e                                                                                                  |
       ]])
@@ -796,8 +785,7 @@ void testFunction () {
       {7:  }{8:  3 }{4:  }{27:// }{4:}                                     }│{7:  }{8:  4 }{4:    }                                       }|
       {7:  }{8:    }{23:-------------------------------------------}│{7:  }{8:  5 }{22:  }                                         }|
       {7:  }{8:  4 }}                                          │{7:  }{8:  6 }}                                           |
-      {7:  }{8:  5 }                                           │{7:  }{8:  7 }                                            |
-      {1:~                                                }│{1:~                                                 }|*11
+      {1:~                                                }│{1:~                                                 }|*12
       {3:Xtest-functional-diff-screen-1.2                  }{2:Xtest-functional-diff-screen-1                    }|
       :e                                                                                                  |
       ]])
@@ -836,8 +824,7 @@ void testFunction () {
       {7:  }{8:  6 }{22:?B                                         }│{7:  }{8:    }{23:--------------------------------------------}|
       {7:  }{8:  7 }{22:?B                                         }│{7:  }{8:    }{23:--------------------------------------------}|
       {7:  }{8:  8 }{22:?C                                         }│{7:  }{8:    }{23:--------------------------------------------}|
-      {7:  }{8:  9 }                                           │{7:  }{8:  4 }                                            |
-      {1:~                                                }│{1:~                                                 }|*9
+      {1:~                                                }│{1:~                                                 }|*10
       {3:Xtest-functional-diff-screen-1.2                  }{2:Xtest-functional-diff-screen-1                    }|
       :e                                                                                                  |
       ]])
@@ -876,8 +863,7 @@ void testFunction () {
       {7:  }{8:  6 }{27:?}{4:B                                         }│{7:  }{8:  2 }{27:!}{4:B                                          }|
       {7:  }{8:  7 }{27:?}{4:C                                         }│{7:  }{8:  3 }{27:!}{4:C                                          }|
       {7:  }{8:  8 }{22:?C                                         }│{7:  }{8:    }{23:--------------------------------------------}|
-      {7:  }{8:  9 }                                           │{7:  }{8:  4 }                                            |
-      {1:~                                                }│{1:~                                                 }|*9
+      {1:~                                                }│{1:~                                                 }|*10
       {3:Xtest-functional-diff-screen-1.2                  }{2:Xtest-functional-diff-screen-1                    }|
       :e                                                                                                  |
       ]])
@@ -1019,8 +1005,7 @@ something
       {7:  }{8:  9 }HIL                                        │{7:  }{8:  9 }HIL                                         |
       {7:  }{8: 10 }common line                                │{7:  }{8: 10 }common line                                 |
       {7:  }{8: 11 }something                                  │{7:  }{8: 11 }something                                   |
-      {7:  }{8: 12 }                                           │{7:  }{8: 12 }                                            |
-      {1:~                                                }│{1:~                                                 }|*6
+      {1:~                                                }│{1:~                                                 }|*7
       {3:Xtest-functional-diff-screen-1.2 [+]              }{2:Xtest-functional-diff-screen-1                    }|
       :1,19diffget                                                                                        |
       ]])
@@ -1093,7 +1078,6 @@ describe('regressions', function()
     clear()
     feed(':set diffopt+=linematch:30<cr>')
     screen = Screen.new(100, 20)
-    screen:attach()
     -- line must be greater than MATCH_CHAR_MAX_LEN
     n.api.nvim_buf_set_lines(0, 0, -1, false, { string.rep('a', 1000) .. 'hello' })
     n.exec 'vnew'
@@ -1105,7 +1089,6 @@ describe('regressions', function()
     clear()
     feed(':set diffopt+=linematch:10<cr>')
     screen = Screen.new(100, 20)
-    screen:attach()
     local lines = {}
     for i = 0, 29 do
       lines[#lines + 1] = tostring(i)
@@ -1151,4 +1134,32 @@ describe('regressions', function()
       },
     }
   end)
+
+  -- oldtest: Test_linematch_3diffs_sanity_check()
+  it('sanity check with 3 diff buffers', function()
+    clear()
+    screen = Screen.new(75, 20)
+    n.api.nvim_buf_set_lines(0, 0, -1, false, { 'abcd', 'def', 'hij' })
+    n.exec('rightbelow vnew')
+    n.api.nvim_buf_set_lines(0, 0, -1, false, { 'defq', 'hijk', 'nopq' })
+    n.exec('rightbelow vnew')
+    n.api.nvim_buf_set_lines(0, 0, -1, false, { 'hijklm', 'nopqr', 'stuv' })
+    n.exec([[
+      set diffopt+=linematch:60
+      windo diffthis | wincmd t
+      call feedkeys("Aq\<esc>")
+      call feedkeys("GAklm\<esc>")
+      call feedkeys("o")
+    ]])
+    screen:expect([[
+      {7:  }{22:abcdq                 }│{7:  }{23:----------------------}│{7:  }{23:-----------------------}|
+      {7:  }{4:def                   }│{7:  }{4:def}{27:q}{4:                  }│{7:  }{23:-----------------------}|
+      {7:  }{4:hijk}{27:lm}{4:                }│{7:  }{4:hijk                  }│{7:  }{4:hijk}{27:lm}{4:                 }|
+      {7:  }{23:----------------------}│{7:  }{4:nopq                  }│{7:  }{4:nopq}{27:r}{4:                  }|
+      {7:  }{4:^                      }│{7:  }{23:----------------------}│{7:  }{27:stuv}{4:                   }|
+      {1:~                       }│{1:~                       }│{1:~                        }|*13
+      {3:[No Name] [+]            }{2:[No Name] [+]            [No Name] [+]            }|
+      {5:-- INSERT --}                                                               |
+    ]])
+  end)
 end)
