@@ -38,7 +38,7 @@ describe('b:changedtick', function()
     -- Somehow undo counts as two changes
     eq(5, changedtick())
   end)
-  it('is present in b: dictionary', function()
+  it('is present in b: dict', function()
     eq(2, changedtick())
     command('let d = b:')
     eq(2, api.nvim_get_var('d').changedtick)
@@ -168,7 +168,7 @@ describe('b:changedtick', function()
     )
     eq(2, changedtick())
   end)
-  it('does not inherit VAR_FIXED when copying dictionary over', function()
+  it('does not inherit VAR_FIXED when copying dict over', function()
     eq(2, changedtick())
     eq('', exec_capture('let d1 = copy(b:)|let d1.changedtick = 42'))
     eq('', exec_capture('let d2 = copy(b:)|unlet d2.changedtick'))
