@@ -1,7 +1,8 @@
-local luv = require('luv')
-local helpers = require('test.functional.helpers')(after_each)
-local clear, command, eval, eq = helpers.clear, helpers.command, helpers.eval, helpers.eq
-local mkdir = helpers.mkdir
+local t = require('test.testutil')
+local n = require('test.functional.testnvim')()
+
+local clear, command, eval, eq = n.clear, n.command, n.eval, t.eq
+local mkdir = t.mkdir
 
 before_each(function()
   clear()
@@ -12,14 +13,14 @@ before_each(function()
 end)
 
 after_each(function()
-  luv.fs_rmdir('test-glob')
+  vim.uv.fs_rmdir('test-glob')
 end)
 
 describe('glob()', function()
   it("glob('.*') returns . and .. ", function()
-    eq({'.', '..'}, eval("glob('.*', 0, 1)"))
+    eq({ '.', '..' }, eval("glob('.*', 0, 1)"))
     -- Do it again to verify scandir_next_with_dots() internal state.
-    eq({'.', '..'}, eval("glob('.*', 0, 1)"))
+    eq({ '.', '..' }, eval("glob('.*', 0, 1)"))
   end)
   it("glob('*') returns an empty list ", function()
     eq({}, eval("glob('*', 0, 1)"))
